
import UIKit


public extension UIStoryboard {
    static var login: UIStoryboard {
        return UIStoryboard(name: "Login", bundle: nil)
    }
    static var main: UIStoryboard {
        return UIStoryboard(name: "Main", bundle: nil)
    }


    public struct Scene {
        
        static var guide: GuideViewController {
            return UIStoryboard.login.instantiateVC(GuideViewController.self)!
        }
        
        static var chooseLan: LanguageViewController {
            return UIStoryboard.login.instantiateVC(LanguageViewController.self)!
        }
        static var chooseCoun: CountryViewController {
            return UIStoryboard.login.instantiateVC(CountryViewController.self)!
        }
        
        static var index: IndexViewController {
            return UIStoryboard.main.instantiateVC(IndexViewController.self)!
        }
        static var list: NewsListViewController {
            return UIStoryboard.main.instantiateVC(NewsListViewController.self)!
        }
        

        static var history: HistoryViewController {
            return UIStoryboard.main.instantiateVC(HistoryViewController.self)!
        }
        static var subs: SubscriptionViewController {
            return UIStoryboard.main.instantiateVC(SubscriptionViewController.self)!
        }
        static var sourcesNews: SourcesNewsViewController {
            return UIStoryboard.main.instantiateVC(SourcesNewsViewController.self)!
        }
        
        static var setting: SettingViewController {
            return UIStoryboard.main.instantiateVC(SettingViewController.self)!
        }

    }
    
}



public extension UIStoryboard {
    
    
    /// Get view controller from storyboard by its class type
    ///
    ///         let profileVC = storyboard!.instantiateVC(ProfileViewController) /* profileVC is of type ProfileViewController */
    ///
    /// Warning: identifier should match storyboard ID in storyboard of identifier class
    func instantiateVC<T>(_ identifier: T.Type) -> T? {
        let storyboardID = String(describing: identifier)
        if let vc = instantiateViewController(withIdentifier: storyboardID) as? T {
            return vc
        } else {
            return nil
        }
    }
    
    /// Get view controller from storyboard by its class type
    ///
    ///         let profileVC = storyboard!.instantiateVC(ProfileViewController) /* profileVC is of type ProfileViewController */
    ///
    func instantiate<T: UIViewController>(controller: T.Type) -> T
        where T: Identifiable {
            return instantiateViewController(withIdentifier: T.identifier) as! T
    }
    
}


public protocol Identifiable {
    
    static var identifier: String { get }
}

public extension Identifiable {
    
    static var identifier: String {
        return String(describing: self)
    }
}
