//
//  Article.swift
//  News
//
//  Created by Qifeng Xie on 10/04/2019.
//  Copyright © 2019 sct. All rights reserved.
//

import Foundation

let countries = ["ae" : "Emiratos Árabes Unidos", "ar": "Argentina", "at":  "Austria", "au": "Australia", "be":"Bélgica",
                 "bg": "Bulgaria", "br": "Brasil", "ca": "Canadá", "ch":"Suiza", "cn":"China", "co":"Colombia", "cu":"Cuba",
                 "cz":  "República Checa", "de":"Alemania", "eg": "Egipto", "fr": "Francia", "gb": "Reino Unido", "gr": "Grecia",
                 "hu": "Hungría", "id": "Indonesia", "ie": "Irlanda", "il": "Israel", "in": "India", "it": "Italia",
                 "jp":"Japón", "kr": "Corea", "lt":"Lituania", "lv": "Letonia", "ma":  "Marruecos", "mx":  "México", "my": "Malasia",
                 "ng": "Nigeria", "nl": "Países Bajos ", "no": "Noruega", "nz":"Nueva Zelanda ", "ph": "Filipinas", "pl": "Polonia",
                 "pt":"Portugal", "ro":"Rumania", "rs": "Serbia", "ru": "Rusia", "sa":"Arabia Saudí", "se":"Suecia", "sg":"Singapur",
                 "si": "Eslovenia", "sk":"Eslovaquia", "th":"Tailandia", "tr":"Turquía", "ua": "Ucrania", "us": "Estados Unidos ",
                 "ve": "Venezuela", "za": "Sudáfrica"]


let categories = ["business": "Negocios", "entertainment": "Entretenimiento", "general": "General", "health": "Salud",
                  "science": "Ciencia",  "sports": "Deportivo", "technology": "Tecnología"]

let languages = ["ar": "Árabe", "de": "Alemán", "en": "Inglés", "es": "Español", "fr": "Francés", "he": "Hebreo",
                 "it": "Italiano", "nl": "Holandés", "no": "Noruego", "pt": "Portugués", "ru": "Ruso", "se":  "Sami del norte",
                 "zh": "Chino"]

fileprivate let topUrl = "https://newsapi.org/v2/top-headlines?apiKey=a4d3f5590453430d8b12ef6e8dae2d2f&pageSize=30&page=%d&country=%@"
fileprivate let topUrl2 = "https://newsapi.org/v2/top-headlines?apiKey=a4d3f5590453430d8b12ef6e8dae2d2f&pageSize=30&page=%d&language=%@"

fileprivate let topWithSourceUrl = "https://newsapi.org/v2/top-headlines?apiKey=a4d3f5590453430d8b12ef6e8dae2d2f&pageSize=30&page=%d&sources=%@"

fileprivate let searchUrl = "https://newsapi.org/v2/everything?apiKey=a4d3f5590453430d8b12ef6e8dae2d2f&pageSize=30&page=1&q=%@"

fileprivate let sourceUrl = "https://newsapi.org/v2/sources?apiKey=a4d3f5590453430d8b12ef6e8dae2d2f"

struct Source: Decodable {
    let id: String?
    let name: String
    
    let description: String?
    let url: String?
    let language: String?
    let country: String?
    
    init(id: String?, name: String) {
        self.id = id
        self.name = name
        
        description = nil
        url = nil
        language = nil
        country = nil
    }
}

struct SourceList: Decodable {
    let status: String
    let sources: [Source]
    
}

struct Article: Decodable {
    let source: Source
    let author: String?
    let title: String
    let url: String
    let urlToImage: String?
    let description: String?
    
    let publishedAt: String
    let content: String?
    
    init(reading: Reading) {
        source = Source(id: reading.source_id, name: reading.source_name!)
        title = reading.title!
        url = reading.url!
        urlToImage = reading.urlToImage
        
        author = nil
        description = nil
        publishedAt = ""
        content = nil
    }
}

struct Newsapi: Decodable {
    let status: String = ""
    let totalResults: Int = 0
    let articles: [Article]
    
    init() {
        articles = []
    }
    
    
    // data from url address
    static func getTopNews(_ page: Int = 1, country: String, cate: String? = nil, completionHandle: @escaping (_ datas: [Article]) -> ()) {
        var str = String(format: topUrl, page, country)
        if let ca = cate {
            str = str + "&" + "category=" + ca
        }
          print("\n\n====Request URL: \n" + str + "\n\n")
        if let url = URL(string: str) {
            let session = URLSession.shared
            // closure containing asynchronous download process
            session.dataTask(with: url) { (data, response, err) in
                guard let jsonData = data else { return }
                do {
                    let decoder = JSONDecoder()
                    let news = try decoder.decode(Newsapi.self, from: jsonData)
                    DispatchQueue.main.async {
                        completionHandle(news.articles)
                    }
                } catch let jsonErr {
                    print("Error decoding JSON", jsonErr)
                }
                }.resume()
            
        }
    }
    
    // data from url address
    static func getTopNews(_ page: Int = 1, language: String, completionHandle: @escaping (_ datas: [Article]) -> ()) {
        if let url = URL(string: String(format: topUrl2, page, language)) {
            //print(url.absoluteString)
            print("\n\n====Request URL: \n" + url.absoluteString + "\n\n")
            
//            let session = URLSession.shared
            let session: URLSession = {
                let config = URLSessionConfiguration.default
                config.timeoutIntervalForRequest = 100
                return URLSession(configuration: config)
            }()
            // closure containing asynchronous download process
            session.dataTask(with: url) { (data, response, err) in
                guard let jsonData = data else { return }
                do {
                    let decoder = JSONDecoder()
                    let news = try decoder.decode(Newsapi.self, from: jsonData)
                    DispatchQueue.main.async {
                        print(news.articles)
                        completionHandle(news.articles)
                    }
                } catch let jsonErr {
                    print("Error decoding JSON", jsonErr)
                }
                }.resume()
            
        }
    }
    
    // data from url address
    static func getSourcesNews(_ page: Int = 1, source_id: String, completionHandle: @escaping (_ datas: [Article]) -> ()) {
        if let url = URL(string: String(format: topWithSourceUrl, page, source_id)) {
            print("\n\n====Request URL: \n" + url.absoluteString + "\n\n")
            let session = URLSession.shared
            // closure containing asynchronous download process
            session.dataTask(with: url) { (data, response, err) in
                guard let jsonData = data else { return }
                do {
                    let decoder = JSONDecoder()
                    let news = try decoder.decode(Newsapi.self, from: jsonData)
                    DispatchQueue.main.async {
                        completionHandle(news.articles)
                    }
                } catch let jsonErr {
                    print("Error decoding JSON", jsonErr)
                }
                }.resume()
            
        }
    }
    
    static func getSearchNews(_ q: String, completionHandle: @escaping (_ datas: [Article]) -> ()) {
        if let url = URL(string: String(format: searchUrl, q)) {
            
            print("\n\n====Request URL: \n" + url.absoluteString + "\n\n")
            let session = URLSession.shared
            // closure containing asynchronous download process
            session.dataTask(with: url) { (data, response, err) in
                guard let jsonData = data else { return }
                do {
                    let decoder = JSONDecoder()
                    let news = try decoder.decode(Newsapi.self, from: jsonData)
                    DispatchQueue.main.async {
                        completionHandle(news.articles)
                    }
                } catch let jsonErr {
                    print("Error decoding JSON", jsonErr)
                }
                }.resume()
            
        }
    }
    
    static func getSources(_ q: String, country: String, completionHandle: @escaping (_ datas: [Source]) -> ()) {
        if let url = URL(string: String(format: searchUrl, country, q)) {
            
            print("\n\n====Request URL: \n" + url.absoluteString + "\n\n")
            let session = URLSession.shared
            // closure containing asynchronous download process
            session.dataTask(with: url) { (data, response, err) in
                guard let jsonData = data else { return }
                do {
                    let decoder = JSONDecoder()
                    let d = try decoder.decode(SourceList.self, from: jsonData)
                    DispatchQueue.main.async {
                        completionHandle(d.sources)
                    }
                } catch let jsonErr {
                    print("Error decoding JSON", jsonErr)
                }
                }.resume()
            
        }
    }
}

