//
//  DBManager.swift
//  News
//
//  Created by Qifeng Xie on 10/04/2019.
//  Copyright © 2019 sct. All rights reserved.
//

import Foundation
import CoreData


class DBManager {
    
    private static let instance = DBManager()
    
    public static var `default`: DBManager {
        return instance
    }
    
    let rssEntity = "Rss"
    
    let readingEntity = "Reading"

    public lazy var context: NSManagedObjectContext = {
        let context = NSManagedObjectContext(concurrencyType: .mainQueueConcurrencyType)
        context.persistentStoreCoordinator = self.persistentStoreCoordinator
        return context
    }()
    
    public lazy var persistentStoreCoordinator: NSPersistentStoreCoordinator = {
        
        let coordinator = NSPersistentStoreCoordinator(managedObjectModel: self.mangerModel)
        let dirURL = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).last
        let fileURL = URL(string: "db.sqlite", relativeTo: dirURL)
        
        do {
            try coordinator.addPersistentStore(ofType: NSSQLiteStoreType, configurationName: nil, at: fileURL, options: nil)
        } catch {
            fatalError("Error configuring persistent store: \(error)")
        }
        return coordinator
    }()
    
    public lazy var mangerModel: NSManagedObjectModel = {
        guard let url = Bundle.main.url(forResource: "News", withExtension: "momd") else {
            fatalError("load db failed")
        }
        guard let model = NSManagedObjectModel(contentsOf: url) else {
            fatalError("load db failed")
        }
        return model
    }()
    

    
    func getRss() -> [Rss] {
        let request = NSFetchRequest<NSFetchRequestResult>(entityName: rssEntity)
        do {
            let result = try context.fetch(request) as! [Rss]
            print(result)
            print(context.registeredObjects)
            return result
        } catch {
            print(error)
            return []
        }
        
    }
    
    func getRssBy(_ id: String) -> [Rss] {
        let request = NSFetchRequest<NSFetchRequestResult>(entityName: rssEntity)
        
        let predicate = NSPredicate.init(format: "id = %@ ", id)
        request.predicate = predicate
        
        do {
            let result = try context.fetch(request) as! [Rss]
            print(result)
            print(context.registeredObjects)
            return result
        } catch {
            print(error)
            return []
        }
    }
    
    func addRss(_ id: String, title: String, desc: String?,  url: String?) {
        let request = NSFetchRequest<NSFetchRequestResult>(entityName: rssEntity)
        let predicate = NSPredicate.init(format: "id = %@ ", id)
        request.predicate = predicate
        
        do {   //save instance
            let result = try context.fetch(request) as! [Rss]
            if result.count > 0 {
                return
            }
            let entity = NSEntityDescription.entity(forEntityName: rssEntity, in: context)
            let object = NSManagedObject(entity: entity!, insertInto: context)
            object.setValue(id, forKey: "id")
            object.setValue(title, forKey: "title")
            object.setValue(desc, forKey: "desc")
            object.setValue(url, forKey: "url")
            
            try context.save()
        } catch  {
            let nserror = error as NSError
            fatalError("Error:\(nserror),\(nserror.userInfo)")
        }
    }
    
    func deleteRss(_ id: String) {
        
        let request = NSFetchRequest<NSFetchRequestResult>(entityName: rssEntity)
        let predicate = NSPredicate.init(format: "id = %@ ", id)
        request.predicate = predicate
        do {
            let result = try context.fetch(request)
            result.forEach({ (person) in
                context.delete(person as! NSManagedObject)
            })
            print(context.deletedObjects)
            if context.hasChanges {
                try context.save()
            }
        } catch {
            print(error)
        }
    }
    

    
    // MARK:
    
    func getReadings(_ url: String? = nil) -> [Reading] {
        let request = NSFetchRequest<NSFetchRequestResult>(entityName: readingEntity)
        if let u = url {
            let predicate = NSPredicate.init(format: "url = %@ ", u)
            request.predicate = predicate
        }
        do {
            let result = try context.fetch(request) as! [Reading]
            print(result)
            print(context.registeredObjects)
            return result
        } catch {
            print(error)
            return []
        }
        
    }
    
    func addReading(_ title: String, url: String, urlToImage: String?, source_id: String?, source_name: String?) {
        let entity = NSEntityDescription.entity(forEntityName: readingEntity, in: context)
        let object = NSManagedObject(entity: entity!, insertInto: context)
        object.setValue(title, forKey: "title")
        object.setValue(url, forKey: "url")
        object.setValue(urlToImage, forKey: "urlToImage")
        object.setValue(source_id, forKey: "source_id")
        object.setValue(source_name, forKey: "source_name")
        
        do {   //save instance
            try context.save()
        } catch  {
            let nserror = error as NSError
            fatalError("Error:\(nserror),\(nserror.userInfo)")
        }
    }
    
    func deleteReading(_ url: String) {
        
        let request = NSFetchRequest<NSFetchRequestResult>(entityName: readingEntity)
        let predicate = NSPredicate.init(format: "url = %@ ", url)
        request.predicate = predicate
        do {
            let result = try context.fetch(request)
            result.forEach({ (person) in
                context.delete(person as! NSManagedObject)
            })
            print(context.deletedObjects)
            if context.hasChanges {
                try context.save()
            }
        } catch {
            print(error)
        }
    }
    

    

}
