//
//  CountryViewController.swift
//  News
//
//  Created by Qifeng Xie on 10/04/2019.
//  Copyright © 2019 sct. All rights reserved.
//

import UIKit

class CountryViewController: MTBaseViewController {
    @IBOutlet weak var tableView: UITableView!
    
    var cs: [String] = ["", ""]
    
    var selectCate: String = ""
    
    override func viewDidLoad() {
        super.viewDidLoad()
        title = "Noticias Por Países"
        
        addNavigationBarLeftButton(self)
        addNavigationBarRightButton(self, action: #selector(go), text: "Hecho")
        
        if let value = UserDefaults.standard.value(forKey: "launch") as? JSONMap {
            if let cs = value["country"] as? [String] {
                self.cs = cs
            }
            if let selectCate = value["cate"] as? String {
                self.selectCate = selectCate
            }
        }
    }

    @objc func go() {
        if cs.filter({$0.count > 0}).count != 2 {
            showMessage("Por favor, seleccionar 2 países", during: 1.5)
            return
        }
        if cs[0] == cs[1] {
            showMessage("Por favor, seleccionar 2 diferentes países", during: 1.5)
            return
        }
        var info: JSONMap = ["country": cs]
        if selectCate.count > 0 {
            info["cate"] = selectCate
        }
        UserDefaults.standard.set(info, forKey: "launch")
        AppDelegate.shared.finishGuide()
    }
}

extension CountryViewController: UITableViewDelegate, UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        return 2
    }
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 60
    }
    
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 40
    }
    
    //TableView format of overview, background
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        let view = UIView(frame: CGRect(x: 0, y: 0, width: tableView.bounds.width, height: 40))
        view.backgroundColor = UIColor(red: 0.98, green: 0.97, blue: 0.94, alpha: 1)
        
        let label = UILabel(frame: CGRect(x: 20, y: 0, width: 280, height: 40))
        label.font = pingFang_SC.regular(13)
        label.textColor = UIColor.darkGray
        if section == 0 {
            label.text = "Seleccionar 2 países"
        } else {
            label.text = "Seleccionar una categoría"
        }
        
        view.addSubview(label)
        return view
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if section == 0 {
            return 2
        } else {
            return categories.keys.count
        }
    }
    
    //TableView cell(row) format
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        if indexPath.section == 0 {
            let cell = tableView.dequeueReusableCell(withIdentifier: "cell1")!
            
            cell.textLabel?.font = pingFang_SC.regular(16)
            cell.textLabel?.textColor = MTColor.title111
            cell.textLabel?.text = indexPath.row == 0 ? "País 1" : "País 2"
            cell.accessoryType = .disclosureIndicator
            cell.detailTextLabel?.font = pingFang_SC.regular(18)
            cell.detailTextLabel?.text = cs.count > indexPath.row ? cs[indexPath.row] : ""
            cell.detailTextLabel?.textColor = MTColor.main
            return cell
        } else {
            let cell = tableView.dequeueReusableCell(withIdentifier: "cell2")!
            
            cell.textLabel?.font = pingFang_SC.regular(16)
            cell.textLabel?.textColor = MTColor.title111
            cell.accessoryType = .none
            
            let values =  Array(categories.values).sorted(by: <)
            
            cell.textLabel?.text = values[indexPath.row]
            
            let c = categories.allKeysForValue(values[indexPath.row])[0]
            if selectCate == c {
                cell.tintColor = MTColor.main
                cell.accessoryType = .checkmark
            }
            
            return cell
        }

        
        
    }
    
    //Jump to document detail scene
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        tableView.cellForRow(at: indexPath)?.setSelected(false, animated: true)
        if indexPath.section == 0 {
            showPopupMenuText("", rows: Array(countries.values).sorted(), selectedText: cs[indexPath.row]) { (index) in
                self.cs[indexPath.row]  = Array(countries.values).sorted()[index]
                self.tableView.reloadData()
            }
        } else {
            let values =  Array(categories.values).sorted(by: <)
            let c = categories.allKeysForValue(values[indexPath.row])[0]
            if selectCate == c {
                selectCate = ""
            } else {
                selectCate = c
            }
            tableView.reloadData()
        }
    }
}
