//
//  IndexViewController.swift
//  News
//
//  Created by Qifeng Xie on 10/04/2019.
//  Copyright © 2019 sct. All rights reserved.
//

import UIKit
import  EasyPeasy
import Kingfisher

class IndexViewController: MTBaseViewController {
    
    
    private let viewPagerNavigationBar: BmoViewPagerNavigationBar = BmoViewPagerNavigationBar()

    
    @IBOutlet weak var viewPager: BmoViewPager!

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        navigationController?.setNavigationBarHidden(false, animated: animated)
        
        viewPagerNavigationBar.easy.layout(CenterX(0), CenterY(0), Width(160), Height(42))
    }
    
    override func viewWillLayoutSubviews() {
        super.viewWillLayoutSubviews()
        viewPagerNavigationBar.easy.layout(CenterX(0), CenterY(0), Width(160), Height(42))
    }
    var titles = ["", ""]
    var items = ["", ""]
    var isCountry: Bool = false
    
    override func viewDidLoad() {
        super.viewDidLoad()

        viewPagerNavigationBar.backgroundColor = .clear
        
        viewPager.dataSource = self
        viewPagerNavigationBar.autoFocus = false
        viewPagerNavigationBar.viewPager = viewPager
        
        addNavigationBarRightButton(self, action: #selector(search), image: UIImage(named: "search")!)
        
        /// 只有这样才能保持居中
        let titleView = UIView(frame: CGRect(x: 0, y: 0, width: 200, height: 44))
        viewPagerNavigationBar.frame = CGRect(x: 20, y: 0, width: 160, height: 42)
        titleView.addSubview(viewPagerNavigationBar)
        navigationItem.titleView = titleView
        
        if let value = UserDefaults.standard.value(forKey: "launch") as? JSONMap {
            if let cs = value["country"] as? [String] {
                self.titles = cs
                let crs = cs.map({countries.allKeysForValue($0)[0]})
                self.items = crs
                isCountry = true
            }
            
            else if let s = value["language"] as? [String] {
                self.titles = s
                let crs = s.map({languages.allKeysForValue($0)[0]})
                self.items = crs
                isCountry = false
            }
        }
    }
    
    
    @objc func search() {
        let vc = self.storyboard?.instantiateVC(SearchViewController.self)
        vc?.hidesBottomBarWhenPushed = true
        self.navigationController?.pushViewController(vc!, animated: true)
    }
    
}

extension IndexViewController: BmoViewPagerDataSource {
    // Optional
    @objc func bmoViewPagerDataSourceNaviagtionBarItemNormalAttributed(_ viewPager: BmoViewPager, navigationBar: BmoViewPagerNavigationBar, forPageListAt page: Int) -> [NSAttributedString.Key : Any]? {
        return [
            NSAttributedString.Key.font : pingFang_SC.regular(14),
            NSAttributedString.Key.foregroundColor : MTColor.title222
            //NSForegroundColorAttributeName : UIColor.groupTableViewBackground
        ]
    }
    @objc func bmoViewPagerDataSourceNaviagtionBarItemHighlightedAttributed(_ viewPager: BmoViewPager, navigationBar: BmoViewPagerNavigationBar, forPageListAt page: Int) -> [NSAttributedString.Key : Any]? {
        return [
            NSAttributedString.Key.font : pingFang_SC.regular(14),
            NSAttributedString.Key.foregroundColor : MTColor.main
        ]
    }
    
    
    func bmoViewPagerDataSourceNaviagtionBarItemHighlightedBackgroundView(_ viewPager: BmoViewPager, navigationBar: BmoViewPagerNavigationBar, forPageListAt page: Int) -> UIView? {
        let view = UnderLineView()
        view.marginX = 15.0
        view.xWidth = 30
        view.lineWidth = 3
        view.strokeColor = MTColor.main
        return view
    }
    
    func bmoViewPagerDataSourceNaviagtionBarItemSize(_ viewPager: BmoViewPager, navigationBar: BmoViewPagerNavigationBar, forPageListAt page: Int) -> CGSize {
        return CGSize(width: 80 , height: navigationBar.frame.size.height)
    }
    
    func bmoViewPagerDataSourceNaviagtionBarItemTitle(_ viewPager: BmoViewPager, navigationBar: BmoViewPagerNavigationBar, forPageListAt page: Int) -> String? {
        return titles[page]
    }
    
    // Required
    func bmoViewPagerDataSourceNumberOfPage(in viewPager: BmoViewPager) -> Int {
        return 2
    }
    
    func bmoViewPagerDataSource(_ viewPager: BmoViewPager, viewControllerForPageAt page: Int) -> UIViewController {
        let vc = UIStoryboard.Scene.list
        if isCountry {
            vc.country = items[page]
        } else {
            vc.language = items[page]
        }
        
        return vc
    }
}


class NewsListViewController: MTBaseViewController {
    
    var datas: [Article] = []
    
    fileprivate var start = 1 {
        didSet {
            self.getDatas()
        }
    }
    fileprivate var totalPages = 0
    
    @IBOutlet weak var tableView: UITableView!
    
    var country: String?
    var category: String?
    
    var language: String?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        
        if let value = UserDefaults.standard.value(forKey: "launch") as? JSONMap {
            if let selectCate = value["cate"] as? String {
                self.category = selectCate
            }
        }
        
        tableView.addRefresh(String.init(describing: self), autoRefresh: false, headerRefresh: { [weak self] in
            self?.refresh()
        }) { [weak self] in
            self?.loadMore()
        }
        
        refresh()
        
    }
    
    /// 刷新数据
    private func refresh() {
        start = 1
    }
    
    private func loadMore() {
        if start < totalPages {
            start += 1
            tableView.es.resetNoMoreData()
        } else {
            tableView.es.noticeNoMoreData()
        }
    }
    
    
    
    func getDatas()  {
        if let c = country {
            MTHUD.showLoading()
            Newsapi.getTopNews(start, country: c, cate: category) { (arts) in
                MTHUD.hide()
                
                // 第一页及刷新，移除数据
                if self.start == 1 {
                    self.datas.removeAll()
                }
                self.datas += arts
                if arts.count == 0 {
                    self.tableView.es.noticeNoMoreData()
                }
                
                self.tableView.reloadData()

                self.tableView.es.stopPullToRefresh()
                self.tableView.es.stopLoadingMore()
            }
        } else if let l = language {
            MTHUD.showLoading()
            Newsapi.getTopNews(start, language: l) { (arts) in
                MTHUD.hide()
                
                // 第一页及刷新，移除数据
                if self.start == 1 {
                    self.datas.removeAll()
                }
                self.datas += arts
                
                self.tableView.reloadData()
                if arts.count == 0 {
                    self.tableView.es.noticeNoMoreData()
                }
                self.tableView.es.stopPullToRefresh()
                self.tableView.es.stopLoadingMore()
            }
        }
    }
    
    
    @IBAction func goDetail(_ index: Int) {
        let detail = datas[index]
        let vc = WebViewController(URL(string:  detail.url)!)
        
        vc.artcle = detail
        vc.hidesBottomBarWhenPushed = true
        self.navigationController?.pushViewController(vc, animated: true)
        
    }

}


extension NewsListViewController : UITableViewDelegate {
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
            return UIScreen.main.bounds.width * 2 / 3
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        tableView.deselectRow(at: indexPath, animated: true)
        goDetail(indexPath.row)
    }
}


extension NewsListViewController : UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return datas.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
            let cell = tableView.dequeueReusableCell(withIdentifier: ArticleCell.className) as! ArticleCell
            cell.info = datas[indexPath.row]
            
            return cell

    }
    
}

class ArticleCell: UITableViewCell {
    
    @IBOutlet weak var titleLabel: UILabel!
    @IBOutlet weak var coverImageView: UIImageView!
    
    override func awakeFromNib() {
        super.awakeFromNib()
        // Initialization code
    }
    
    var info: Article? {
        didSet {
            
            titleLabel.text = info?.title
            if let url = info?.urlToImage {
                coverImageView.kf.setImage(with: URL(string: url))
            }
        }
    }
    
    
}
