//
//  LanguageViewController.swift
//  News
//
//  Created by Qifeng Xie on 10/04/2019.
//  Copyright © 2019 sct. All rights reserved.
//


import UIKit

class LanguageViewController: MTBaseViewController {
    @IBOutlet weak var tableView: UITableView!
    
    var selectedLang: [String] = ["" , ""]
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        title = "Noticias Por Idiomas"
        addNavigationBarLeftButton(self)
        addNavigationBarRightButton(self, action: #selector(go), text: "Hecho")
        
        if let value = UserDefaults.standard.value(forKey: "launch") as? JSONMap {
            if let s = value["language"] as? [String] {
                self.selectedLang = s
            }
        }
    }
    
    @objc func go() {
        // save
        if selectedLang.filter({$0.count > 0}).count != 2 {
            showMessage("Por favor, seleccionar 2 idiomas", during: 1.5)
            return
        }
        if selectedLang[0] == selectedLang[1] {
            showMessage("Por favor, seleccionar 2 diferentes idiomas", during: 1.5)
            return
        }
        
        let info = ["language": selectedLang]
        UserDefaults.standard.set(info, forKey: "launch")
        
        AppDelegate.shared.finishGuide()
    }
}

extension LanguageViewController: UITableViewDelegate, UITableViewDataSource {

    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 60
    }
    
    func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 40
    }
    
    //TableView format of overview, background
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        let view = UIView(frame: CGRect(x: 0, y: 0, width: tableView.bounds.width, height: 40))
        view.backgroundColor = UIColor(red: 0.98, green: 0.97, blue: 0.94, alpha: 1)
        
        let label = UILabel(frame: CGRect(x: 20, y: 0, width: 280, height: 40))
        label.font = pingFang_SC.regular(13)
        label.textColor = UIColor.darkGray
        label.text = "Seleccionar 2 Idiomas"
        view.addSubview(label)
        return view
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 2
    }
    
    //TableView cell(row) format
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: "cell1")!
        
        if indexPath.row == 0 {
            cell.textLabel?.text = "Idioma 1"
        } else {
            cell.textLabel?.text = "Idioma 2"
        }
        
        cell.textLabel?.font = pingFang_SC.regular(16)
        //color yellow
        cell.textLabel?.textColor = MTColor.title111
        cell.detailTextLabel?.textColor = MTColor.main
        cell.detailTextLabel?.font = pingFang_SC.regular(18)
        
        if indexPath.row < selectedLang.count {
            cell.detailTextLabel?.text = selectedLang[indexPath.row]
        }
        
        return cell
    }
    
    //Jump to document detail scene
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        tableView.cellForRow(at: indexPath)?.setSelected(false, animated: true)
        
        showPopupMenuText("", rows: Array(languages.values).sorted(), selectedText: selectedLang[indexPath.row]) { (index) in
            self.selectedLang[indexPath.row]  = Array(languages.values).sorted()[index]
            self.tableView.reloadData()
        }
    }
}
