//
//  SourcesNewsViewController.swift
//  News
//
//  Created by Qifeng Xie on 10/04/2019.
//  Copyright © 2019 sct. All rights reserved.
//

import UIKit

class SourcesNewsViewController: MTBaseViewController {

    var datas: [Article] = []
    
    fileprivate var start = 1 {
        didSet {
            self.getDatas()
        }
    }
    fileprivate var totalPages = 0
    
    @IBOutlet weak var tableView: UITableView!
    
    var source_id: String?
    
    @objc func saveRss(_ sender: UIButton) {
        if sender.isSelected {
            DBManager.default.deleteRss(source_id!)
        } else {
            DBManager.default.addRss(source_id!, title: title ?? "", desc: nil, url: nil)
        }
        sender.isSelected = !sender.isSelected
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        addNavigationBarLeftButton(self)
        
        tableView.addRefresh(String.init(describing: self), autoRefresh: false, headerRefresh: { [weak self] in
            self?.refresh()
        }) { [weak self] in
            self?.loadMore()
        }
        
        refresh()
        
        if let id = source_id {
            let ss = DBManager.default.getRssBy(id)
            
            //let suggestButtonContainer = UIView(frame: CGRect(x: 0, y: 0, width: 44, height: 44))
            
            let leftButton2: UIButton = UIButton(frame: CGRect(x: 60, y: 0, width: 32, height: 32))
            leftButton2.setImage(UIImage(named: "rss"), for: .normal)
            leftButton2.setImage(UIImage(named: "rss_active"), for: .selected)
            leftButton2.imageEdgeInsets = UIEdgeInsets(top: 8, left: 8, bottom: 8, right: 8)
            leftButton2.isSelected = ss.count > 0
            leftButton2.addTarget(self, action: #selector(saveRss(_:)), for: .touchUpInside)
            leftButton2.transform = CGAffineTransform(translationX: 15, y: 0)   //15 刚好
            //suggestButtonContainer.addSubview(leftButton2)
            
            //let suggestButtonItem = UIBarButtonItem(customView: suggestButtonContainer)
            
            self.navigationItem.rightBarButtonItem = UIBarButtonItem(customView:leftButton2)
        }
    }
    
    /// 刷新数据
    private func refresh() {
        start = 1
    }
    
    private func loadMore() {
        if start < totalPages {
            start += 1
            tableView.es.resetNoMoreData()
        } else {
            tableView.es.noticeNoMoreData()
        }
    }
    
    
    
    func getDatas()  {
        if let s = source_id {
            MTHUD.showLoading()
            Newsapi.getSourcesNews(start, source_id: s) { (arts) in
                MTHUD.hide()
                
                // 第一页及刷新，移除数据
                if self.start == 1 {
                    self.datas.removeAll()
                }
                self.datas += arts
                if arts.count == 0 {
                    self.tableView.es.noticeNoMoreData()
                }
                
                self.tableView.reloadData()
                
                self.tableView.es.stopPullToRefresh()
                self.tableView.es.stopLoadingMore()
            }
        }
    }
    
    
    @IBAction func goDetail(_ index: Int) {
        let detail = datas[index]
        let vc = WebViewController(URL(string:  detail.url)!)
        
        vc.artcle = detail
        vc.hidesBottomBarWhenPushed = true
        self.navigationController?.pushViewController(vc, animated: true)
        
    }
    
}


extension SourcesNewsViewController : UITableViewDelegate {
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UIScreen.main.bounds.width * 2 / 3
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        tableView.deselectRow(at: indexPath, animated: true)
        goDetail(indexPath.row)
    }
}


extension SourcesNewsViewController : UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return datas.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: ArticleCell.className) as! ArticleCell
        cell.info = datas[indexPath.row]
        
        return cell
        
    }
    
}
